require 'webrick'
require 'webrick/httpstatus'
require './session'
require "amrita2/template"

include Amrita2

module AmritaHandler

  class AmritaHandlerUtil
    def self.dynamic_class_load(class_name)
      class_name_down = class_name.downcase
      exact_name = nil
      
      Module.constants.each do |c|
        if c.downcase == class_name_down
          exact_name = c
          break
        end
      end
#    raise NameError.new("undefined class name", class_name) unless exact_name
      clazz = Module::const_get(exact_name)
      raise NameError.new("undefined class name", class_name) unless clazz.instance_of?(Class)
      clazz
    end  
  
    def self.build_processor(clazz, req)
      instance = clazz.new
      keys = instance.reader_methods & req.query.keys
      keys.each do |k|
        instance.__send__("#{k}=", req.query[k])
      end
      instance
    end
    
    def self.inject_parameter(object, req)
      req.query.keys.each do |k|
        if m = /(.*)_(\d+)$/.match(k)
          prop = m.to_a[1]
          id = m.to_a[2].to_i
          begin
            obj2 = ObjectSpace::_id2ref(id)
            obj2.__send__("#{prop}=", req.query[k])
          rescue
            nil
          end
        end
      end
          
      keys = object.writable_properties & req.query.keys
      keys.each do |k|
        object.__send__("#{k}=", req.query[k])
      end
    end
  end


  class AmritaViewHandler < WEBrick::HTTPServlet::DefaultFileHandler
  
    def do_GET(req, res)
      local_script = @local_path.sub(/\.html$/, '')
      local_script += '.rb'
      if File.exist?(local_script)
        req.class.module_eval %Q{ attr_accessor :session }
        req.session = WEBrick::Session.new(req, res)

        class_name = File.basename(local_script, '.rb')
        load(local_script)
  
        #clazz = AmritaHandlerUtil::dynamic_class_load(class_name)
        #instance = AmritaHandlerUtil::build_processor(clazz, req)
        instance = $registry[class_name.downcase.intern]
        AmritaHandlerUtil::inject_parameter(instance, req)
        
        if instance.respond_to?(:preprocess)
          instance.__send__(:preprocess)
        end
        
        opt = {
          :check_true => true, 
          :replace_attr => true,
          :use_original_element => true,
          :delete_id => true, 
          :key_attr_name => :id
        }
        tmpl = TemplateFile.new(@local_path) # do
        #  Amrita2::define_template_spec(opt)
        #end
#        tmpl.spec.instance_eval do
#          @option[:use_original_element] = true
#        end
        out = ""
        tmpl.expand(out, instance)
        res.body = out
        req.session.update
      else
        super
      end
      
    end
    
    def do_POST(req, res)
      res.header['Allow'] = 'GET'
      raise HTTPStatus::MethodNotAllowed
    end
  
  end
  
  
  class AmritaActionHandler < WEBrick::HTTPServlet::DefaultFileHandler
  
    def do_POST(req, res)
      req.class.module_eval %Q{ attr_accessor :session }
      req.session = WEBrick::Session.new(req, res)

      class_name = File::basename(@local_path, '.rb')
      load(@local_path)
      
      #clazz = AmritaHandlerUtil::dynamic_class_load(class_name)
      #instance = AmritaHandlerUtil::build_processor(clazz, req)
      instance = $registry[class_name.downcase.intern]
      AmritaHandlerUtil::inject_parameter(instance, req)
      
      result = nil
      req.query.keys.each do |k|
        if /^action:/.match(k)
          method = k[7..-1].to_sym
          result2 = instance.__send__(method) if instance.respond_to?(method)
          result = result2 if result2
        end
      end
      if instance.respond_to?(:action)
        result2 = instance.__send__(:action)
        result = result2 if result2
      end
      
      req.session.update
      forwarding = File::dirname(req.path) + '/'
      forwarding << result + '.html' if result
      # 303 See Other ĝ|Cg
      res.set_redirect(HTTPStatus::SeeOther, forwarding)
    end
  
    def do_GET(req, res)
      res.header['Allow'] = 'POST'
      raise HTTPStatus::MethodNotAllowed
    end
  end

  CONFIG = {
    :NondisclosureName => [".ht*", "*~"],
    :FancyIndexing     => true,
    :HandlerTable      => {'html'=>AmritaViewHandler, 'rb'=>AmritaActionHandler},
    :HandlerCallback   => nil,
    :DirectoryCallback => nil,
    :FileCallback      => nil,
    :UserDir           => nil,
    :AcceptableLanguages => ['en', 'ja']  # ["en", "ja", ... ]
  }

end

module Amrita2
#  class Template
#    def get_option
#      ret = {
#        :check_true=>true, 
#        :replace_attr=>false,
#        :use_original_element => true,
#        :delete_id => (not @keep_id), 
#        :key_attr_name=>( @amrita_id or :id)
#      }
#      ret
#    end
#  end

  def self.make_default_spec1(e, option, spec_stack)
    key_attr = option[:key_attr_name].to_s
    id_ = e.attributes[key_attr]
    if id_
      sym = id_.intern

      option2 = option.clone
      if e.name == 'input' or e.name == 'select'
        option2[:use_original_element] = true
        option2[:inject_attr_value] = true
      else
        option2.delete(:use_original_element)
        option2.delete(:inject_attr_value)
      end
      s = Core::DynamicElementSpec.new(sym, nil, option2)

      s.parent = spec_stack.first
      spec_stack.first.children << s
      spec_stack.unshift(s)
      e.elements.each do |c|
        make_default_spec1(c, option, spec_stack)
      end
      spec_stack.shift
    else
      e.elements.each do |c|
        make_default_spec1(c, option, spec_stack)
      end
    end
  end
end

class Object
  def writable_properties
    ms = self.methods
    ms.reject! {|m| m[-1, 1] != '='}
    ms.each do |m|
      m.chop!
    end
    ms
  end
end

=begin
class Module
#alias :_orig_attr_reader :attr_reader
#def attr_reader(*arg)
#  
#end
  def attr_input(*arg)
    arg.each do |a|
      method = a.to_s
      module_eval %Q{
        def #{method}(m = nil)
          m.#{method}('', :value=>@#{method}) if m
          @#{method}
        end
      }
    end
  end
  def attr_action_input(*arg)
    arg.each do |a|
      method = a.to_s
      module_eval %Q{
        def #{method}(m = nil)
          m.#{method}('', :value=>@#{method}, :name=>"#{method}_\#{__id__}") if m
          @#{method}
        end
      }
    end
  end
end
=end

